@@ -13,7 +13,7 @@ from skued import diffread
 
 # Get all proper subclasses of AbstractRawDataset
 # to build a loading menu
-from .. import AbstractRawDataset, __version__, __author__, __license__
+from .. import AbstractRawDataset, __author__, __license__, __version__
 from ..plugins import PLUGIN_DIR, install_plugin
 from .angular_average_dialog import AngularAverageDialog
 from .calibrate_q_dialog import QCalibratorDialog
@@ -25,6 +25,7 @@ from .powder_viewer import PowderViewer
 from .processing_dialog import ProcessingDialog
 from .qbusyindicator import QBusyIndicator
 from .symmetrize_dialog import SymmetrizeDialog
+from .update import update_available, update_in_background
 
 image_folder = join(dirname(__file__), 'images')
 
@@ -69,7 +70,7 @@ class Iris(QtWidgets.QMainWindow, metaclass = ErrorAware):
 
         self.controller.processed_dataset_loaded_signal.connect(lambda b: self.viewer_stack.setCurrentWidget(self.processed_viewer))
         self.controller.processed_dataset_loaded_signal.connect(self.controls.diffraction_dataset_controls.setVisible)
-        self.controller.processed_dataset_loaded_signal.connect(self.controls.stack.setVisible)
+        self.controller.processed_dataset_loaded_signal.connect(self.controls.metadata_and_notes_widget.setVisible)
 
         self.controller.powder_dataset_loaded_signal.connect(lambda b: self.viewer_stack.setCurrentWidget(self.powder_viewer))
         self.controller.powder_dataset_loaded_signal.connect(self.controls.powder_diffraction_dataset_controls.setVisible)
@@ -103,7 +104,8 @@ class Iris(QtWidgets.QMainWindow, metaclass = ErrorAware):
         # Viewers -------------------------------------------------------------
         self.raw_data_viewer = pg.ImageView(parent = self, name = 'Raw data')
         self.raw_data_viewer.resize(self.raw_data_viewer.maximumSize())
-        self.controller.raw_data_signal.connect(lambda obj: self.raw_data_viewer.clear() if obj is None else self.raw_data_viewer.setImage(obj))
+        self.controller.raw_data_signal.connect(
+            lambda obj: self.raw_data_viewer.clear() if obj is None else self.raw_data_viewer.setImage(obj, autoLevels = False))
 
         self.processed_viewer = ProcessedDataViewer(parent = self)
         self.processed_viewer.peak_dynamics_roi_signal.connect(self.controller.time_series)
@@ -140,7 +142,7 @@ class Iris(QtWidgets.QMainWindow, metaclass = ErrorAware):
         # Dynamically add an option for each implementation of AbstractRawDataset
         # Note : because of dynamical nature of these bindings,
         # it must be done in a separate method
-        for cls in AbstractRawDataset.implementations:
+        for cls in sorted(AbstractRawDataset.implementations, key = lambda cls: cls.__name__):
             self._create_load_raw(cls, load_raw_submenu)
 
         self.load_dataset_action = QtWidgets.QAction(QtGui.QIcon(join(image_folder, 'locator.png')), '& Load dataset', self)
@@ -199,26 +201,6 @@ class Iris(QtWidgets.QMainWindow, metaclass = ErrorAware):
         self.promote_to_powder_action.triggered.connect(self.launch_promote_to_powder_dialog)
         self.controller.processed_dataset_loaded_signal.connect(self.promote_to_powder_action.setEnabled)
 
-        self.show_diff_peak_dynamics_action = QtWidgets.QAction('& Show/hide peak dynamics', self)
-        self.show_diff_peak_dynamics_action.setCheckable(True)
-        self.show_diff_peak_dynamics_action.toggled.connect(self.processed_viewer.toggle_peak_dynamics)
-        self.controller.processed_dataset_loaded_signal.connect(self.show_diff_peak_dynamics_action.setEnabled)
-
-        self.show_diff_relative_action = QtWidgets.QAction('& Toggle relative dynamics', self)
-        self.show_diff_relative_action.setCheckable(True)
-        self.show_diff_relative_action.toggled.connect(self.controller.enable_averaged_relative)
-        self.controller.processed_dataset_loaded_signal.connect(self.show_diff_relative_action.setEnabled)
-
-        self.show_powder_relative_action = QtWidgets.QAction('& Toggle relative dynamics', self)
-        self.show_powder_relative_action.setCheckable(True)
-        self.show_powder_relative_action.toggled.connect(self.controller.enable_powder_relative)
-        self.controller.powder_dataset_loaded_signal.connect(self.show_powder_relative_action.setEnabled)
-
-        self.toggle_powder_background_action = QtWidgets.QAction('& Remove baseline', self)
-        self.toggle_powder_background_action.setCheckable(True)
-        self.toggle_powder_background_action.toggled.connect(self.controller.powder_background_subtracted)
-        self.controller.powder_dataset_loaded_signal.connect(self.toggle_powder_background_action.setEnabled)
-
         self.update_metadata_action = QtWidgets.QAction(QtGui.QIcon(join(image_folder, 'save.png')), '& Update dataset metadata', self)
         self.update_metadata_action.triggered.connect(self.launch_metadata_edit_dialog)
         self.controller.processed_dataset_loaded_signal.connect(self.update_metadata_action.setEnabled)
@@ -241,7 +223,35 @@ class Iris(QtWidgets.QMainWindow, metaclass = ErrorAware):
         self.diffraction_dataset_menu.addAction(self.recompute_angular_averages)
         self.diffraction_dataset_menu.addAction(self.calibrate_scattvector_action)
 
+        ###################
         # Display options
+        self.show_diff_peak_dynamics_action = QtWidgets.QAction('& Show/hide peak dynamics', self)
+        self.show_diff_peak_dynamics_action.setCheckable(True)
+        self.show_diff_peak_dynamics_action.toggled.connect(self.processed_viewer.toggle_peak_dynamics)
+        self.controller.processed_dataset_loaded_signal.connect(self.show_diff_peak_dynamics_action.setEnabled)
+
+        self.show_diff_peak_dynamics_bounds_action = QtWidgets.QAction('& Show/hide peak dynamics bounds', self)
+        self.show_diff_peak_dynamics_bounds_action.setCheckable(True)
+        self.show_diff_peak_dynamics_bounds_action.toggled.connect(self.processed_viewer.toggle_roi_bounds_text)
+        self.controller.processed_dataset_loaded_signal.connect(self.show_diff_peak_dynamics_bounds_action.setEnabled)
+
+        self.show_diff_relative_action = QtWidgets.QAction('& Toggle relative dynamics', self)
+        self.show_diff_relative_action.setCheckable(True)
+        self.controller.relative_averaged_enable_signal.connect(self.show_diff_relative_action.setChecked)
+        self.show_diff_relative_action.toggled.connect(self.controller.enable_averaged_relative)
+        self.controller.processed_dataset_loaded_signal.connect(self.show_diff_relative_action.setEnabled)
+
+        self.show_powder_relative_action = QtWidgets.QAction('& Toggle relative dynamics', self)
+        self.show_powder_relative_action.setCheckable(True)
+        self.controller.relative_powder_enable_signal.connect(self.show_powder_relative_action.setChecked)
+        self.show_powder_relative_action.toggled.connect(self.controller.enable_powder_relative)
+        self.controller.powder_dataset_loaded_signal.connect(self.show_powder_relative_action.setEnabled)
+
+        self.toggle_powder_background_action = QtWidgets.QAction('& Remove baseline', self)
+        self.toggle_powder_background_action.setCheckable(True)
+        self.controller.powder_bgr_enable_signal.connect(self.toggle_powder_background_action.setChecked)
+        self.toggle_powder_background_action.toggled.connect(self.controller.powder_background_subtracted)
+        self.controller.powder_dataset_loaded_signal.connect(self.toggle_powder_background_action.setEnabled)
 
         self.display_options_menu = self.menu_bar.addMenu('&Display')
         self.diffraction_dataset_display_options_menu = self.display_options_menu.addMenu('& Diffraction display options')
@@ -252,6 +262,7 @@ class Iris(QtWidgets.QMainWindow, metaclass = ErrorAware):
 
         # Assemble submenus
         self.diffraction_dataset_display_options_menu.addAction(self.show_diff_peak_dynamics_action)
+        self.diffraction_dataset_display_options_menu.addAction(self.show_diff_peak_dynamics_bounds_action)
         self.diffraction_dataset_display_options_menu.addAction(self.show_diff_relative_action)
 
         self.powder_dataset_display_options_menu.addAction(self.show_powder_relative_action)
@@ -259,7 +270,6 @@ class Iris(QtWidgets.QMainWindow, metaclass = ErrorAware):
 
         ###################
         # Helps and misc operations
-
         self.about_action = QtWidgets.QAction('& About', self)
         self.about_action.triggered.connect(self.show_about)
 
@@ -272,12 +282,18 @@ class Iris(QtWidgets.QMainWindow, metaclass = ErrorAware):
         self.report_issue_action = QtWidgets.QAction(QtGui.QIcon(join(image_folder, 'revert.png')), '& Report issue', self)
         self.report_issue_action.triggered.connect(lambda: QtGui.QDesktopServices.openUrl(QtCore.QUrl("https://github.com/LaurentRDC/iris-ued/issues/new")))
 
+        self.update_action = QtWidgets.QAction('& Update to latest version', self)
+        self.update_action.triggered.connect(self.update_iris)
+        self.update_action.setEnabled(self.update_available()[0])
+
         self.help_menu = self.menu_bar.addMenu('&Help')
         self.help_menu.addAction(self.about_action)
         self.help_menu.addSeparator()
         self.help_menu.addAction(self.launch_documentation_action)
         self.help_menu.addAction(self.goto_repository_action)
         self.help_menu.addAction(self.report_issue_action)
+        self.help_menu.addSeparator()
+        self.help_menu.addAction(self.update_action)
 
         ###################
         # Layout
@@ -438,29 +454,92 @@ class Iris(QtWidgets.QMainWindow, metaclass = ErrorAware):
         self.controller.close_raw_dataset()
         self.restart_signal.emit()
     
+    @QtCore.pyqtSlot()
+    def update_iris(self):
+        """ Update iris-ued package in the background then restart """
+        explanation = "You are about to update iris. All datasets will be closed, and iris will restart.\n\nAre you sure you want to do this?"
+        
+        # This method cannot be reached unless an update is available
+        # Therefore, we only check for the latest version.
+        _, latest_version = update_available()
+        answer = QtWidgets.QMessageBox.warning(self, 'Updating iris', explanation,
+                                               QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
+                                               QtWidgets.QMessageBox.No)
+
+        if answer == QtWidgets.QMessageBox.No:
+            return
+
+        update_in_background()
+
+        self.controller.close_dataset()
+        self.controller.close_raw_dataset()
+        self.restart_signal.emit()
+
+    @staticmethod
+    def update_available():
+        """ Check if iris-ued is outdated. In case of 
+        connection error, pretend iris-ued is latest version """
+        try:
+            return update_available()
+        except ConnectionError:
+            return False, __version__
+    
     @QtCore.pyqtSlot()
     def show_about(self):
         """ Show the About information """
-        import h5py
-        import sys
-        import skued
-        import pyqtgraph
 
-        about = f"""<h2>About Iris</h2>
-        Iris is both a Python library and a GUI program for the exploration of ultrafast electron diffraction data. <br>
-        <b>License</b>: {__license__}                    <br>
-        <b>Author</b>: {__author__}                      <br>
-        <b>Install location</b>: {Path(__file__).parent} <br> 
+        return QtWidgets.QMessageBox.about(self, 'About Iris', make_about_string())
 
-        <h3>Versions</h3>                                <br>
-        <b>Python version</b>: {sys.version}             <br>
-        <b>iris version</b>: {__version__}               <br>
-        <b>Qt version</b>: {QtCore.qVersion()}           <br>
-        <b>PyQtGraph version</b>: {pyqtgraph.__version__}<br>
-        <b>scikit-ued version</b>: {skued.__version__}   <br>
-        <b>h5py version</b>: {h5py.version.version}      <br>
-        <b>HDF5 version</b>: {h5py.version.hdf5_version} <br>
+def make_about_string():
+    import h5py
+    import sys
+    import skued
+    import pyqtgraph
+    import npstreams
 
-        <h3>Installed plug-ins</h3>
-        {'<br>'.join(cls.__name__ for cls in AbstractRawDataset.implementations)} """
-        return QtWidgets.QMessageBox.about(self, 'About Iris', about)
\ No newline at end of file
+    python_version = '{}.{}.{}'.format(*sys.version_info[:3])
+
+    return f"""<h2>About Iris</h2>
+    Iris is both a Python library and a GUI program for the exploration of ultrafast electron diffraction data. <br>
+    <b>License</b>: {__license__}                    <br>
+    <b>Author</b>: {__author__}                      <br>
+    <b>Install location</b>: {Path(__file__).parent} <br> 
+
+    <h3>Versions</h3>
+    <table style="width:100%">
+        <tr>
+            <td style="padding-right:75">Python</td>
+            <td align="right">{python_version}</td>
+        </tr>
+        <tr>
+            <td style="padding-right:75">Iris</td>
+            <td align="right">{__version__}</td>
+        </tr>
+        <tr>
+            <td style="padding-right:75">Qt</td>
+            <td align="right">{QtCore.qVersion()}</td>
+        </tr>
+        <tr>
+            <td style="padding-right:75">PyQtGraph</td>
+            <td align="right">{pyqtgraph.__version__}</td>
+        </tr>
+        <tr>
+            <td style="padding-right:75">scikit-ued</td>
+            <td align="right">{skued.__version__}</td>
+        </tr>
+        <tr>
+            <td style="padding-right:75">npstreams</td>
+            <td align="right">{npstreams.__version__}</td>
+        </tr>
+        <tr>
+            <td style="padding-right:75">HDF5</td>
+            <td align="right">{h5py.version.hdf5_version}</td>
+        </tr>
+        <tr>
+            <td style="padding-right:75">h5py</td>
+            <td align="right">{h5py.version.version}</td>
+        </tr>
+    </table>
+
+    <h3>Installed plug-ins</h3>
+    {'<br>'.join(cls.__name__ for cls in sorted(AbstractRawDataset.implementations,  key = lambda cls: cls.__name__))} """
